@@ -6,6 +6,7 @@
 #include <errno.h>
 #include <libintl.h>
 #include <stdlib.h>
+#include <libgen.h>
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -45,13 +46,22 @@ gint dbus_req_handler (const gchar *interface, const gchar *method,
         GArray *arguments, gpointer data, osso_rpc_t *retval);
 void exit_event_handler (gboolean die_now, gpointer data);
 
-static gchar *_app_file_chooser (Application *app, GtkFileChooserAction act) {
+gchar *app_file_chooser (Application *app, GtkFileChooserAction act) {
     GtkWidget *dialog;
     gchar *filename = NULL;
     dialog = hildon_file_chooser_dialog_new (app->window, act);
+    if (app->save_folder)
+        gtk_file_chooser_set_current_folder (
+                GTK_FILE_CHOOSER (dialog), app->save_folder);
+    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
     gtk_widget_show_all (GTK_WIDGET (dialog));
     if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
         filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+        if (filename && filename[0]) {
+            if (app->save_folder)
+                g_free (app->save_folder);
+            app->save_folder = dirname (g_strdup (filename));
+        }
     }
     gtk_widget_destroy (dialog);
     return filename;
@@ -112,14 +122,23 @@ static gboolean delayed_open_location (OpenLocation *ol) {
     return FALSE;
 }
 
-void app_set_playing (Application *app, bool p) {
+void app_set_playing (Application *app, int p) {
     app->playing = p;
-    if (p) {
+    if (PLAYING == p) {
         gtk_widget_hide (GTK_WIDGET (app->tb_play));
         gtk_widget_show (GTK_WIDGET (app->tb_pause));
+        gtk_widget_hide (GTK_WIDGET (app->tb_save));
+        gtk_widget_show (GTK_WIDGET (app->tb_stop));
     } else {
         gtk_widget_hide (GTK_WIDGET (app->tb_pause));
         gtk_widget_show (GTK_WIDGET (app->tb_play));
+        if (NOT_PLAYING == p) {
+            gtk_widget_hide (GTK_WIDGET (app->tb_stop));
+            gtk_widget_show (GTK_WIDGET (app->tb_save));
+        } else {
+            gtk_widget_hide (GTK_WIDGET (app->tb_save));
+            gtk_widget_show (GTK_WIDGET (app->tb_stop));
+        }
     }
 }
 
@@ -169,7 +188,7 @@ void app_normal_screen (Application *app) {
 
 static void callback_file_open (GtkButton * button, Application * app) {
     gchar *filename = NULL;
-    filename = _app_file_chooser(app, GTK_FILE_CHOOSER_ACTION_OPEN);
+    filename = app_file_chooser(app, GTK_FILE_CHOOSER_ACTION_OPEN);
     if (filename) {
         app_open_location (filename, app);
         g_free (filename);
@@ -189,7 +208,7 @@ static void callback_file_go (GtkButton *button, Application *app) {
 
 static void callback_file_save_as (GtkButton * button, Application * app) {
     gchar *filename = NULL;
-    filename = _app_file_chooser (app, GTK_FILE_CHOOSER_ACTION_SAVE);
+    filename = app_file_chooser (app, GTK_FILE_CHOOSER_ACTION_SAVE);
     if (filename) {
         gchar * info;
         if (saveFile (app->control, filename)) {
@@ -238,6 +257,7 @@ gboolean locationbarKeyEvent (GtkWidget *w, GdkEventKey *e, Application *app) {
     (void)w;
     switch (e->keyval) {
         case GDK_Return:
+        case GDK_KP_Enter:
             callback_file_go (NULL, app);
             return true;
     }
@@ -251,7 +271,7 @@ bool cb_window_expose (GtkWidget *w, GdkEventExpose *ev, Application *app) {
     if (!first_time_exposed) {
         first_time_exposed = true;
         gtk_paned_set_position (app->hsplit_view, HSPLIT);
-        gtk_paned_set_position (app->vsplit_view, 200);
+        gtk_paned_set_position (app->vsplit_view, VSPLIT);
         gtk_widget_hide (GTK_WIDGET (app->tb_progress));
         gtk_widget_hide (GTK_WIDGET (app->tb_item_playlist_commit));
     }
@@ -270,7 +290,6 @@ static void create_toolbar (Application * app) {
     GtkToolItem *tb_open;
     GtkToolItem *tb_prev;
     GtkToolItem *tb_next;
-    GtkToolItem *tb_stop;
     GtkToolItem *tb_go;
     GtkToolItem *tb_separator;
 
@@ -283,7 +302,8 @@ static void create_toolbar (Application * app) {
     app->tb_play = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_PLAY);
     app->tb_pause = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_PAUSE);
     tb_next = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_NEXT);
-    tb_stop = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_STOP);
+    app->tb_stop = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_STOP);
+    app->tb_save = gtk_tool_button_new_from_stock (GTK_STOCK_SAVE);
     tb_go = gtk_tool_button_new (gtk_image_new_from_icon_name(
                 "qgn_toolb_browser_gobutton", HILDON_ICON_SIZE_TOOLBAR), NULL);
     app->tb_item_playlist_commit = GTK_WIDGET (gtk_tool_button_new (
@@ -320,7 +340,8 @@ static void create_toolbar (Application * app) {
     gtk_toolbar_insert(GTK_TOOLBAR(app->main_toolbar), app->tb_play, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(app->main_toolbar), app->tb_pause, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(app->main_toolbar), tb_next, -1);
-    gtk_toolbar_insert(GTK_TOOLBAR(app->main_toolbar), tb_stop, -1);
+    gtk_toolbar_insert(GTK_TOOLBAR(app->main_toolbar), app->tb_stop, -1);
+    gtk_toolbar_insert(GTK_TOOLBAR(app->main_toolbar), app->tb_save, -1);
 
     /* Add signal lister to "Close" button */
     g_signal_connect (G_OBJECT (tb_go), "clicked",
@@ -335,7 +356,9 @@ static void create_toolbar (Application * app) {
         G_CALLBACK (cb_playlist_play), app->control);
     g_signal_connect (G_OBJECT (tb_next), "clicked",
         G_CALLBACK (cb_playlist_next), app->control);
-    g_signal_connect (G_OBJECT (tb_stop), "clicked",
+    g_signal_connect (G_OBJECT (app->tb_stop), "clicked",
+        G_CALLBACK (cb_playlist_stop), app->control);
+    g_signal_connect (G_OBJECT (app->tb_save), "clicked",
         G_CALLBACK (cb_playlist_stop), app->control);
     g_signal_connect (G_OBJECT (app->tb_item_playlist_commit), "clicked",
         G_CALLBACK (cb_playlist_edit_commit), app);
@@ -346,6 +369,7 @@ static void create_toolbar (Application * app) {
     gtk_widget_show_all (GTK_WIDGET (app->main_toolbar));
     gtk_widget_hide (GTK_WIDGET (app->tb_progress));
     gtk_widget_hide (GTK_WIDGET (app->tb_pause));
+    gtk_widget_hide (GTK_WIDGET (app->tb_stop));
     hildon_window_add_toolbar (HILDON_WINDOW (app->window),
             GTK_TOOLBAR(app->main_toolbar));
 }
@@ -650,7 +674,7 @@ int main (int argc, char* argv[]) {
             GTK_SIGNAL_FUNC (cb_keyPressed), (void *)app);
     /* Begin the main application */
     gtk_paned_set_position (app->hsplit_view, HSPLIT);
-    gtk_paned_set_position (app->vsplit_view, 200);
+    gtk_paned_set_position (app->vsplit_view, VSPLIT);
     gtk_widget_show_all (GTK_WIDGET (main_window));
 
     osso_mime_set_cb (app->osso_context, cb_openUrls, app);
