@@ -57,7 +57,8 @@ public:
 
 class KMPLAYER_NO_EXPORT Item : public Mrl {
 public:
-    KDE_NO_CDTOR_EXPORT Item (NodePtr & d) : Mrl (d, id_node_item) {}
+    KDE_NO_CDTOR_EXPORT Item (NodePtr & d)
+        : Mrl (d, id_node_item), timer (NULL) {}
     NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "item"; }
     PlayType playType () { return cached_play_type; }
@@ -65,9 +66,9 @@ public:
     void closed ();
     void activate ();
     void deactivate ();
-    bool handleEvent (Event *event);
+    void *message (MessageType msg, void *content=NULL);
     NodePtrW enclosure;
-    EventPtrW timer;
+    Posting *timer;
 };
 
 class KMPLAYER_NO_EXPORT Enclosure : public Mrl {
