@@ -29,7 +29,7 @@ struct TransTypeInfo;
 
 namespace KMPlayer {
 
-class ImageMedia;
+class ImageActor;
 
 typedef std::vector <String> StringList;
 
@@ -82,21 +82,33 @@ class KMPLAYER_NO_EXPORT Runtime {
 public:
     enum TimingState {
         timings_reset = 0, timings_began,
-        timings_started, timings_paused, timings_stopped
+        timings_started, timings_paused, timings_stopped, timings_freezed
+    };
+    enum Fill {
+        fill_default, fill_inherit, fill_remove, fill_freeze,
+        fill_hold, fill_transition, fill_auto
     };
     enum DurationTime { begin_time = 0, duration_time, end_time, durtime_last };
     enum Duration {
-        dur_infinite = -1, dur_timer = 0, dur_media,
-        dur_activated, dur_inbounds, dur_outbounds,
-        dur_end, dur_start, dur_last_dur
+        dur_infinite = -1,
+        dur_timer = (int) MsgEventTimer,
+        dur_activated = (int) MsgEventClicked,
+        dur_inbounds = (int) MsgEventPointerInBounds,
+        dur_outbounds = (int) MsgEventPointerOutBounds,
+        dur_start = (int) MsgEventStarted,
+        dur_end = (int) MsgEventStopped,
+        dur_media = (int)MsgMediaFinished,
+        dur_transition,
+        dur_last_dur
     };
-    Runtime (NodePtr e);
+    Runtime (Element *e);
     ~Runtime ();
     /**
      * Called when element is pulled in scope, from Node::activate()
      */
-    void begin ();
-    void beginAndStart (); // skip start timer (if any)
+    void start ();
+    void finish ();
+    void startAndBeginNode (); // skip start timer (if any)
     /**
      * Reset all data, called from end() and init()
      */
@@ -105,7 +117,8 @@ public:
     TimingState state () const { return timingstate; }
     void propagateStop (bool forced);
     void propagateStart ();
-    void processEvent (unsigned int event);
+    void *message (MessageType msg, void *content=NULL);
+    void processEvent (MessageType msg);
     /**
      * Duration items, begin/dur/end, length information or connected element
      */
@@ -119,6 +132,9 @@ public:
     bool started () const {
         return timingstate == timings_started || timingstate == timings_paused;
     }
+    bool active () const {
+        return timingstate >= timings_started && timingstate != timings_stopped;
+    }
     void stopped ();
     KDE_NO_EXPORT DurationItem & beginTime () { return durations[begin_time]; }
     KDE_NO_EXPORT DurationItem & durTime () { return durations[duration_time]; }
@@ -127,22 +143,30 @@ public:
     TimingState timingstate;
     TimingState unpaused_state;
     int repeat_count;
-    EventPtrW start_timer;
-    EventPtrW duration_timer;
-    EventPtrW started_timer;
-    EventPtrW stopped_timer;
+    NodeRefListPtr m_StartListeners;        // Element about to be started
+    NodeRefListPtr m_StartedListeners;      // Element is started
+    NodeRefListPtr m_StoppedListeners;      // Element stopped
+    Posting *begin_timer;
+    Posting *duration_timer;
+    Posting *started_timer;
+    Posting *stopped_timer;
     NodePtrW paused_by;
+    unsigned int start_time;
+    unsigned int finish_time;
     unsigned int paused_time;
+    Fill fill;
+    Fill fill_def;
+    Fill fill_active;
+    Element *element;
 private:
     void setDurationItem (DurationTime item, const String & val);
-    NodePtrW element;
 };
 
 class KMPLAYER_NO_EXPORT MouseListeners {
 public:
     MouseListeners();
 
-    NodeRefListPtr listeners (unsigned int event_id);
+    NodeRefList *receivers (MessageType msg);
 
     NodeRefListPtr m_ActionListeners;      // mouse clicked
     NodeRefListPtr m_OutOfBoundsListeners; // mouse left
@@ -205,9 +229,8 @@ public:
     void activate ();
     void deactivate ();
     void closed ();
-    void childDone (NodePtr child);
     bool expose () const;
-    bool handleEvent (Event *event);
+    void *message (MessageType msg, void *content=NULL);
     void accept (Visitor *v) { v->visit (this); }
     void jump (const String & id);
     static Smil * findSmilNode (Node * node);
@@ -225,10 +248,26 @@ public:
     NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "head"; }
     void closed ();
-    void childDone (NodePtr child);
+    void *message (MessageType msg, void *content=NULL);
     bool expose () const;
 };
 
+/**
+ * Defines region layout, should reside below 'head' element
+ */
+class KMPLAYER_NO_EXPORT Layout : public Element {
+public:
+    Layout (NodePtr & d);
+    NodePtr childFromTag (const String & tag);
+    KDE_NO_EXPORT const char * nodeName () const { return "layout"; }
+    void closed ();
+    void *message (MessageType msg, void *content=NULL);
+    void accept (Visitor *v) { v->visit (this); }
+    bool expose () const { return false; }
+
+    NodePtrW root_layout;
+};
+
 /**
  * Base class for SMIL::Region, SMIL::RootLayout and SMIL::Layout
  */
@@ -237,12 +276,12 @@ public:
     enum ShowBackground { ShowAlways, ShowWhenActive };
 
     ~RegionBase ();
-    bool expose () const { return false; }
-    void activate ();
-    void childDone (NodePtr child);
-    void deactivate ();
+    virtual bool expose () const { return false; }
+    virtual void activate ();
+    virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (Event *event);
+    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void accept (Visitor *v) { v->visit (this); }
     /**
      * repaints region, calls scheduleRepaint(x,y,w,h) on view
      */
@@ -256,9 +295,8 @@ public:
     virtual void updateDimensions ();
     void boundsUpdate (); // recalculates and repaint old and new bounds
 
-    virtual Surface *surface ();
     SurfacePtrW region_surface;
-    ImageMedia *bg_image;
+    ImageActor *bg_image;
     CalculatedSizer sizes;
 
     Single x, y, w, h;     // unscaled values
@@ -267,6 +305,11 @@ public:
     String background_image;
     ShowBackground show_background;
     Fit fit;
+    NodeRefListPtr m_AttachedMediaTypes;   // active attached mediatypes
+    /**
+     * boolean for check if pointerEntered/pointerLeft should be called by View
+     */
+    bool has_mouse;
 protected:
     RegionBase (NodePtr & d, short id);
     PostponePtr postpone_lock;               // pause while loading bg image
@@ -274,24 +317,16 @@ protected:
 };
 
 /**
- * Defines region layout, should reside below 'head' element
+ * Represents the root area for the other regions
  */
-class KMPLAYER_NO_EXPORT Layout : public RegionBase {
+class KMPLAYER_NO_EXPORT RootLayout : public RegionBase {
 public:
-    Layout (NodePtr & d);
-    NodePtr childFromTag (const String & tag);
-    KDE_NO_EXPORT const char * nodeName () const { return "layout"; }
-    void activate ();
+    KDE_NO_CDTOR_EXPORT RootLayout (NodePtr & d)
+        : RegionBase (d, id_node_root_layout) {}
     void closed ();
-    virtual void accept (Visitor *);
-    /**
-     * recursively calculates dimensions of this and child regions
-     */
-    virtual void updateDimensions ();
-    virtual Surface *surface ();
-
-    NodePtrW root_layout;
-    NodePtrW default_region;
+    void *message (MessageType msg, void *content=NULL);
+    KDE_NO_EXPORT const char * nodeName () const { return "root-layout"; }
+    void updateDimensions ();
 };
 
 /**
@@ -303,27 +338,11 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return "region"; }
     NodePtr childFromTag (const String & tag);
     void calculateBounds (Single w, Single h);
-    virtual NodeRefListPtr listeners (unsigned int event_id);
-    virtual void accept (Visitor *);
-    /**
-     * boolean for check if pointerEntered/pointerLeft should be called by View
-     */
-    bool has_mouse;
-    NodeRefListPtr m_AttachedMediaTypes;   // active attached mediatypes
+    void *message (MessageType msg, void *content=NULL);
 private:
     MouseListeners mouse_listeners;
 };
 
-/**
- * Represents the root area for the other regions
- */
-class KMPLAYER_NO_EXPORT RootLayout : public RegionBase {
-public:
-    KDE_NO_CDTOR_EXPORT RootLayout (NodePtr & d)
-        : RegionBase (d, id_node_root_layout) {}
-    KDE_NO_EXPORT const char * nodeName () const { return "root-layout"; }
-};
-
 /**
  * Represents a regPoint element for alignment inside regions
  */
@@ -388,79 +407,27 @@ public:
 };
 
 /**
- * Base for all SMIL media elements having begin/dur/end/.. attributes
+ * Abstract base for the group elements (par/seq/excl/..)
  */
-class KMPLAYER_NO_EXPORT TimedMrl : public Mrl {
+class KMPLAYER_NO_EXPORT GroupBase : public Element {
 public:
-    enum Fill {
-        fill_default, fill_inherit, fill_remove, fill_freeze,
-        fill_hold, fill_transition, fill_auto
-    };
-    ~TimedMrl ();
-    void closed ();
-    void activate ();
-    void begin ();
-    void pause ();
+    ~GroupBase ();
+    NodePtr childFromTag (const String & tag);
+    PlayType playType () { return play_type_none; }
+    void parseParam (const TrieString &name, const String &value);
+    void init ();
     void finish ();
+    void activate ();
     void deactivate ();
     void reset ();
     bool expose () const { return false; }
-    void childBegan (NodePtr child);
-    void childDone (NodePtr child);
-    virtual bool handleEvent (Event *event);
-    virtual NodeRefListPtr listeners (unsigned int event_id);
-    KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
-    void init ();
-    virtual void parseParam (const TrieString &, const String &);
-    Runtime * runtime ();
-    static bool isTimedMrl (const Node *n);
-    static bool keepContent (Node *n);
-    static Fill getDefaultFill (NodePtr n);
-    unsigned int begin_time;
-    unsigned int finish_time;
-    Fill fill;
-    Fill fill_def;
-    Fill fill_active;
-protected:
-    TimedMrl (NodePtr & d, short id);
-
-    NodeRefListPtr m_StartListeners;        // Element about to be started
-    NodeRefListPtr m_StartedListeners;      // Element is started
-    NodeRefListPtr m_StoppedListeners;      // Element stopped
-    Runtime * m_runtime;
-    bool inited;
-};
-
-KDE_NO_EXPORT inline Runtime * TimedMrl::runtime () {
-    if (!m_runtime)
-        m_runtime = new Runtime (this);
-    return m_runtime;
-}
-
-KDE_NO_EXPORT inline bool TimedMrl::isTimedMrl (const Node *n) {
-    return n &&
-        n->id >= id_node_first_timed_mrl &&
-        n->id <= id_node_last_timed_mrl;
-}
-
-/**
- * Abstract base for the group elements (par/seq/excl/..)
- */
-class KMPLAYER_NO_EXPORT GroupBase : public TimedMrl {
-public:
-    KDE_NO_CDTOR_EXPORT ~GroupBase () {}
-    NodePtr childFromTag (const String & tag);
-    PlayType playType () { return play_type_none; }
-    void finish ();
-    void activate ();
-    void begin ();
-    void deactivate ();
-    bool handleEvent (Event *);
+    void *message (MessageType msg, void *content=NULL);
     void setJumpNode (NodePtr);
+    Runtime *runtime;
 protected:
-    KDE_NO_CDTOR_EXPORT GroupBase (NodePtr & d, short id) : TimedMrl (d, id) {}
+    GroupBase (NodePtr & d, short id);
     NodePtrW jump_node;
-    PostponePtr postpone_lock;
+    bool inited;
 };
 
 /**
@@ -472,7 +439,8 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return "par"; }
     void begin ();
     void reset ();
-    void childDone (NodePtr child);
+    void *message (MessageType msg, void *content=NULL);
+    KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 };
 
 /**
@@ -483,7 +451,9 @@ public:
     KDE_NO_CDTOR_EXPORT Seq (NodePtr & d) : GroupBase(d, id_node_seq) {}
     KDE_NO_EXPORT const char * nodeName () const { return "seq"; }
     void begin ();
-    void childDone (NodePtr child);
+    void *message (MessageType msg, void *content=NULL);
+    KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
+    ConnectionPtr starting_connection;
 protected:
     KDE_NO_CDTOR_EXPORT Seq (NodePtr & d, short id) : GroupBase(d, id) {}
 };
@@ -508,8 +478,8 @@ public:
     NodePtr childFromTag (const String & tag);
     void begin ();
     void deactivate ();
-    void childDone (NodePtr child);
-    virtual bool handleEvent (Event *event);
+    void *message (MessageType msg, void *content=NULL);
+    KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 
     typedef ListNode <ConnectionPtr> ConnectionStoreItem;
     List <ConnectionStoreItem> started_event_list;
@@ -529,7 +499,7 @@ public:
     NodePtr childFromTag (const String & tag);
     void init ();
     void parseParam (const TrieString &, const String &);
-    void childDone (NodePtr child);
+    void *message (MessageType msg, void *content=NULL);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 
     enum { PeersStop, PeersPause, PeersDefer, PeersNever } peers;
@@ -549,7 +519,7 @@ public:
     void begin ();
     void deactivate ();
     void reset ();
-    void childDone (NodePtr child);
+    void *message (MessageType msg, void *content=NULL);
     NodePtrW chosenOne;
 };
 
@@ -573,7 +543,7 @@ public:
     Anchor (NodePtr & d);
     KDE_NO_CDTOR_EXPORT ~Anchor () {}
     void activate ();
-    void childDone (NodePtr child);
+    void *message (MessageType msg, void *content=NULL);
     KDE_NO_EXPORT const char * nodeName () const { return "a"; }
     NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
@@ -587,7 +557,7 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return (const char *) tag; }
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
     void parseParam (const TrieString & name, const String & value);
-    NodeRefListPtr listeners (unsigned int event_id);
+    void *message (MessageType msg, void *content=NULL);
     SizeType * coords;
     int nr_coords;
     const String tag;
@@ -597,7 +567,7 @@ public:
 /**
  * Abstract base for the MediaType classes (video/audio/text/img/..)
  */
-class KMPLAYER_NO_EXPORT MediaType : public TimedMrl {
+class KMPLAYER_NO_EXPORT MediaType : public Mrl {
 public:
     MediaType (NodePtr & d, const String & t, short id);
     ~MediaType ();
@@ -612,17 +582,18 @@ public:
     virtual void undefer ();
     virtual void begin ();
     virtual void finish ();
-    virtual void childDone (NodePtr child);
+    virtual void reset ();
     virtual Surface *getSurface (Mrl *mrl);
     /* (new) sub-region or NULL if not displayed */
-    Surface *surface ();
     void resetSurface ();
     SRect calculateBounds ();
     void boundsUpdate (); // recalculates and repaint old and new bounds
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (Event *event);
-    NodeRefListPtr listeners (unsigned int event_id);
+    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void accept (Visitor *v) { v->visit (this); }
+    bool expose () const { return false; }
 
+    Runtime *runtime;
     SurfacePtrW sub_surface;
     NodePtrW external_tree; // if src points to playlist, the resolved top node
     NodePtrW trans_in;
@@ -634,10 +605,10 @@ public:
     Fit fit;
     int opacity;
     unsigned int bitrate;
-    unsigned int trans_step;
-    unsigned int trans_steps;
-    EventPtrW trans_timer;
-    EventPtrW trans_out_timer;
+    unsigned int trans_start_time;
+    unsigned int trans_end_time;
+    float trans_gain;
+    Posting *trans_out_timer;
     enum { sens_opaque, sens_transparent, sens_percentage } sensitivity;
     bool trans_out_active;
     bool has_mouse;
@@ -655,6 +626,7 @@ protected:
     ConnectionPtr region_attach;           // attached to region
     ConnectionPtr document_postponed;      // pause audio/video accordantly
     PostponePtr postpone_lock;
+    bool inited;
 };
 
 class KMPLAYER_NO_EXPORT AVMediaType : public MediaType {
@@ -663,12 +635,12 @@ public:
     NodePtr childFromTag (const String & tag);
     virtual void begin ();
     virtual void undefer ();
+    virtual void finish ();
     virtual void endOfFile ();
     virtual void accept (Visitor *);
     virtual bool expose () const;
     virtual void parseParam (const TrieString &, const String &);
     virtual void clipStart ();
-    virtual void clipStop ();
 };
 
 class KMPLAYER_NO_EXPORT ImageMediaType : public MediaType {
@@ -680,7 +652,7 @@ public:
     virtual void begin ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
-    virtual bool handleEvent (Event *event);
+    virtual void *message (MessageType msg, void *content=NULL);
     void dataArrived ();
 };
 
@@ -692,7 +664,7 @@ public:
     virtual void begin ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
-    virtual bool handleEvent (Event *event);
+    virtual void *message (MessageType msg, void *content=NULL);
     void dataArrived ();
 
     String font_name;
@@ -715,12 +687,18 @@ public:
     virtual void accept (Visitor *);
 };
 
-class KMPLAYER_NO_EXPORT AnimateGroup : public TimedMrl {
+class KMPLAYER_NO_EXPORT AnimateGroup : public Element {
 public:
-    KDE_NO_CDTOR_EXPORT ~AnimateGroup () {}
+    ~AnimateGroup ();
+    virtual void init ();
+    virtual void activate ();
     virtual void finish ();
     virtual void deactivate ();
+    virtual void reset ();
+    virtual bool expose () const { return false; }
     virtual void parseParam (const TrieString & name, const String & value);
+    virtual void *message (MessageType msg, void *content=NULL);
+    Runtime *runtime;
 protected:
     void restoreModification ();
     NodePtr targetElement ();
@@ -729,6 +707,7 @@ protected:
     TrieString changed_attribute;
     String change_to;
     int modification_id;
+    bool inited;
 };
 
 class KMPLAYER_NO_EXPORT Set : public AnimateGroup {
@@ -747,12 +726,12 @@ public:
     virtual void finish ();
     virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (Event *event);
+    virtual void *message (MessageType msg, void *content=NULL);
     virtual void accept (Visitor *v) { v->visit (this); }
     KDE_NO_EXPORT const char * nodeName () const { return "animate"; }
     PlayType playType () { return play_type_none; }
 
-    EventPtrW anim_timer;
+    Posting *anim_timer;
 private:
     void applyStep ();
     bool timerTick();
@@ -770,6 +749,10 @@ private:
 
 class KMPLAYER_NO_EXPORT AnimateMotion : public AnimateGroup {
 public:
+    struct Point2D {
+        float x;
+        float y;
+    };
     AnimateMotion (NodePtr & d);
     ~AnimateMotion ();
     virtual void init ();
@@ -777,14 +760,14 @@ public:
     virtual void finish ();
     virtual void deactivate ();
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual bool handleEvent (Event *event);
+    virtual void *message (MessageType msg, void *content=NULL);
     virtual void accept (Visitor *v) { v->visit (this); }
     KDE_NO_EXPORT const char * nodeName () const { return "animateMotion"; }
     PlayType playType () { return play_type_none; }
 
-    EventPtrW anim_timer;
+    Posting *anim_timer;
 private:
-    bool timerTick();
+    bool timerTick (unsigned int cur_time);
     bool checkTarget (Node *n);
     bool setInterval ();
     void applyStep ();
@@ -796,13 +779,14 @@ private:
     String change_by;
     StringList values;
     float *keytimes;
+    Point2D *spline_table;
     int keytime_count;
     StringList splines;
     float control_point[4];
-    unsigned int steps;
-    unsigned int cur_step;
     unsigned int keytime_steps;
     unsigned int interval;
+    unsigned int interval_start_time;
+    unsigned int interval_end_time;
     SizeType begin_x, begin_y;
     SizeType cur_x, cur_y;
     SizeType delta_x, delta_y;
