@@ -32,12 +32,14 @@
 #include "kmplayer_rss.h"
 #include "kmplayer_smil.h"
 #include "kmplayer_xspf.h"
-#include "mediaobject.h"
+#include "actor.h"
 
 #ifdef SHAREDPTR_DEBUG
 int shared_data_count;
 #endif
 
+#define TIMEOUT_OFF -1
+
 using namespace KMPlayer;
 
 //-----------------------------------------------------------------------------
@@ -93,15 +95,17 @@ TextStream & operator << (TextStream & out, const XMLStringlet & txt) {
 //-----------------------------------------------------------------------------
 
 KDE_NO_CDTOR_EXPORT
-Connection::Connection (NodeRefListPtr ls, NodePtr node, NodePtr inv)
+Connection::Connection (NodeRefList *ls, Node *node, Node *inv)
  : connectee (inv), listeners (ls) {
     if (listeners) {
         NodeRefItemPtr nci = new NodeRefItem (node);
-        listeners->append (nci);
+        ls->append (nci);
         listen_item = nci;
     }
 }
 
+//-----------------------------------------------------------------------------
+
 KDE_NO_EXPORT void Connection::disconnect () {
     if (listen_item && listeners)
         listeners->remove (listen_item);
@@ -111,9 +115,11 @@ KDE_NO_EXPORT void Connection::disconnect () {
 
 //-----------------------------------------------------------------------------
 
-KMPLAYER_NO_CDTOR_EXPORT
-TimerEvent::TimerEvent (int ms, unsigned eid)
- : Event (NULL, event_timer), event_id (eid), milli_sec (ms), interval (false) {}
+KDE_NO_CDTOR_EXPORT TimerPosting::TimerPosting (int ms, unsigned eid)
+ : Posting (NULL, MsgEventTimer),
+   event_id (eid),
+   milli_sec (ms),
+   interval (false) {}
 
 //-----------------------------------------------------------------------------
 
@@ -175,8 +181,12 @@ void Node::defer () {
 
 void Node::undefer () {
     if (state == state_deferred) {
-        setState (state_activated);
-        activate ();
+        if (firstChild () && firstChild ()->state > state_init) {
+            state = state_began;
+        } else {
+            setState (state_activated);
+            activate ();
+        }
     } else
         warningLog() <<"Node::undefer () call on not defered element"<< endl;
 }
@@ -185,7 +195,7 @@ void Node::finish () {
     if (active ()) {
         setState (state_finished);
         if (m_parent)
-            m_parent->childDone (this);
+            document ()->post (m_parent, new Posting (this, MsgChildFinished));
         else
             deactivate (); // document deactivates itself on finish
     } else
@@ -203,7 +213,7 @@ void Node::deactivate () {
             break; // remaining not yet activated
     }
     if (need_finish && m_parent)
-        m_parent->childDone (this);
+        document ()->post (m_parent, new Posting (this, MsgChildFinished));
 }
 
 void Node::reset () {
@@ -219,21 +229,6 @@ void Node::reset () {
     }
 }
 
-void Node::childBegan (NodePtr /*child*/) {
-}
-
-void Node::childDone (NodePtr child) {
-    //kdDebug () << nodeName () << " Node::childDone" << endl;
-    if (unfinished ()) {
-        if (child->state == state_finished)
-            child->deactivate ();
-        if (child->nextSibling ())
-            child->nextSibling ()->activate ();
-        else
-            finish (); // we're done
-    }
-}
-
 void Node::clear () {
     clearChildren ();
 }
@@ -392,19 +387,30 @@ void Node::opened () {}
 
 void Node::closed () {}
 
-NodeRefListPtr Node::listeners (unsigned int /*event_id*/) {
-    return NodeRefListPtr ();
+void *Node::message (MessageType msg, void *content) {
+    if (MsgChildFinished == msg) {
+        Posting *post = (Posting *) content;
+        if (unfinished ()) {
+            if (post->source->state == state_finished)
+                post->source->deactivate ();
+            if (post->source && post->source->nextSibling ())
+                post->source->nextSibling ()->activate ();
+            else
+                finish (); // we're done
+        }
+        return NULL;
+    }
+    if (MsgStartQueryMessage < msg && MsgEndQueryMessage > msg)
+        return NULL;
+    return MsgUnhandled;
 }
 
-bool Node::handleEvent (Event * /*event*/) { return false; }
-
-KDE_NO_EXPORT void Node::propagateEvent (Event *event) {
-    EventPtr store (event);
-    NodeRefListPtr nl = listeners (event->id ());
+KDE_NO_EXPORT void Node::deliver (MessageType msg, void *content) {
+    NodeRefList *nl = nodeMessageReceivers (this, msg);
     if (nl)
         for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ())
             if (c->data)
-                c->data->handleEvent (event);
+                c->data->message (msg, content);
 }
 
 void Node::accept (Visitor * v) {
@@ -412,8 +418,8 @@ void Node::accept (Visitor * v) {
 }
 
 KDE_NO_EXPORT
-ConnectionPtr Node::connectTo (NodePtr node, unsigned int evt_id) {
-    NodeRefListPtr nl = listeners (evt_id);
+ConnectionPtr Node::connectTo (Node *node, MessageType msg) {
+    NodeRefList *nl = nodeMessageReceivers (this, msg);
     if (nl)
         return ConnectionPtr (new Connection (nl, node, this));
     return ConnectionPtr ();
@@ -575,6 +581,10 @@ void Element::setAttributes (AttributeListPtr attrs) {
     m_attributes = attrs;
 }
 
+void Element::accept (Visitor * v) {
+    v->visit (this);
+}
+
 //-----------------------------------------------------------------------------
 
 Attribute::Attribute (const TrieString & n, const String & v)
@@ -599,15 +609,15 @@ static bool hasMrlChildren (const NodePtr & e) {
 
 Mrl::Mrl (NodePtr & d, short id)
     : Element (d, id), cached_ismrl_version (~0),
-      media_object (NULL),
+      actor (NULL),
       aspect (0), repeat (0),
       view_mode (SingleMode),
       resolved (false), bookmarkable (true),
       audio_only (false) {}
 
 Mrl::~Mrl () {
-    if (media_object)
-        media_object->destroy ();
+    if (actor)
+        actor->dismiss ();
 }
 
 Node::PlayType Mrl::playType () {
@@ -682,9 +692,9 @@ void Mrl::begin () {
             if (linkNode ()->unfinished ())
                 setState (state_began);
         } else if (!src.isEmpty ()) {
-            if (!media_object)
-                media_object = document ()->notify_listener->mediaManager()->createMedia (MediaManager::AudioVideo, this);
-            if (media_object->play ())
+            if (!actor)
+                actor = document ()->notify_listener->actorAgent()->hire (ActorAgent::AudioVideo, this);
+            if (actor->play ())
                 setState (state_began);
             else
                 deactivate ();
@@ -694,14 +704,14 @@ void Mrl::begin () {
 }
 
 void Mrl::defer () {
-    if (media_object)
-        media_object->pause ();
+    if (actor)
+        actor->pause ();
     Node::defer ();
 }
 
 void Mrl::undefer () {
-    if (media_object) {
-        media_object->unpause ();
+    if (actor) {
+        actor->unpause ();
         setState (state_began);
     } else {
         Node::undefer ();
@@ -709,9 +719,9 @@ void Mrl::undefer () {
 }
 
 void Mrl::deactivate () {
-    if (media_object) {
-        media_object->destroy ();
-        media_object = NULL;
+    if (actor) {
+        actor->dismiss ();
+        actor = NULL;
     }
     Node::deactivate ();
 }
@@ -725,12 +735,8 @@ Surface *Mrl::getSurface (Mrl *mrl) {
     return NULL;
 }
 
-bool Mrl::handleEvent (Event *) {
-    return false;
-}
-
 void Mrl::parseParam (const TrieString & para, const String & val) {
-    if (para == StringPool::attr_src && !src.hasPrefix ("#")) {
+    if (para == StringPool::attr_src && !src.startsWith ("#")) {
         String abs = absolutePath ();
         if (abs != src)
             src = val;
@@ -775,9 +781,12 @@ void Surface::markDirty () {
 
 //-----------------------------------------------------------------------------
 
-EventData::EventData (Node *t, Event *e, EventData *n)
+EventData::EventData (Node *t, Posting *e, EventData *n)
  : target (t), event (e), next (n) {}
 
+EventData::~EventData () {
+    delete event;
+}
 //-----------------------------------------------------------------------------
 
 Postpone::Postpone (NodePtr doc) : m_doc (doc) {
@@ -802,7 +811,7 @@ Document::Document (const String & s, PlayListNotify * n)
    event_queue (NULL),
    paused_queue (NULL),
    cur_event (NULL),
-   cur_timeout (-1) {
+   cur_time_interval (TIMEOUT_OFF) {
     m_doc = m_self; // just-in-time setting fragile m_self to m_doc
     src = s;
     if (!s.isEmpty ())
@@ -852,8 +861,7 @@ void Document::dispose () {
 
 void Document::activate () {
     first_event_time.tv_sec = 0;
-    last_paint_time.tv_sec = 0;
-    cur_timeout_time.tv_sec = 0;
+    cur_time_interval = TIMEOUT_OFF;
     last_event_time = 0;
     Mrl::activate ();
 }
@@ -871,15 +879,16 @@ void Document::undefer () {
 
 void Document::reset () {
     Mrl::reset ();
+    if (notify_listener && TIMEOUT_OFF != cur_time_interval) {
+        notify_listener->setTimeout (-1);
+        cur_time_interval = TIMEOUT_OFF;
+    }
     if (event_queue) {
-        if (notify_listener)
-            notify_listener->setTimeout (-1);
         while (event_queue) {
             EventData *ed = event_queue;
             event_queue = ed->next;
             delete ed;
         }
-        cur_timeout = -1;
     }
     postpone_lock = 0L;
 }
@@ -895,6 +904,14 @@ static inline void addTime (struct timeval & tv, int ms) {
     tv.tv_usec = (tv.tv_usec + ms*1000) % 1000000;
 }
 
+KDE_NO_CDTOR_EXPORT UpdateEvent::UpdateEvent (Document *doc, unsigned int skip)
+ : skipped_time (skip) {
+    struct timeval tv;
+    doc->timeOfDay (tv);
+    cur_event_time = doc->last_event_time;
+}
+
+//-----------------------------------------------------------------------------
 /*static inline void subtractTime (struct timeval & tv, int ms) {
     int sec = ms / 1000;
     int msec = ms % 1000;
@@ -912,25 +929,26 @@ void Document::timeOfDay (struct timeval & tv) {
     if (!first_event_time.tv_sec) {
         first_event_time = tv;
         last_event_time = 0;
-    } else
-        last_event_time = diffTime (tv, first_event_time) / 100;
+    } else {
+        last_event_time = diffTime (tv, first_event_time);
+    }
 }
 
-static bool postponedSensible (Event *e) {
-    return e->id () == event_timer ||
-        e->id () == event_started ||
-        e->id () == event_stopped;
+static bool postponedSensible (MessageType msg) {
+    return msg == MsgEventTimer ||
+        msg == MsgEventStarted ||
+        msg == MsgEventStopped;
 }
 
-void Document::insertEvent (Node *n, Event *e, const struct timeval &tv) {
+void Document::insertPosting (Node *n, Posting *e, const struct timeval &tv) {
     if (!notify_listener)
         return;
-    bool postponed_sensible = postponedSensible (e);
+    bool postponed_sensible = postponedSensible (e->message);
     EventData *prev = NULL;
     EventData *ed = event_queue;
     for (; ed; ed = ed->next) {
         int diff = diffTime (ed->timeout, tv);
-        bool psens = postponedSensible (ed->event.ptr ());
+        bool psens = postponedSensible (ed->event->message);
         if ((diff > 0 && postponed_sensible == psens) || (!postponed_sensible && psens))
             break;
         prev = ed;
@@ -944,58 +962,39 @@ void Document::insertEvent (Node *n, Event *e, const struct timeval &tv) {
     //debugLog () << "setTimeout " << ms << " at:" << pos << " tv:" << tv.tv_sec << "." << tv.tv_usec << endl;
 }
 
-void Document::setNextTimeout (const struct timeval &n) {
+void Document::setNextTimeout (const struct timeval &now) {
     if (!cur_event) {              // if we're not processing events
         int timeout = 0x7FFFFFFF;
-        static const int off = -1;
-        struct timeval now = n;
         if (event_queue && active () &&
-                (!postpone_ref || !postponedSensible (event_queue->event.ptr ())))
+                (!postpone_ref || !postponedSensible (event_queue->event->message)))
             timeout = diffTime (event_queue->timeout, now);
-        timeout = 0x7FFFFFFF != timeout ? (timeout > 0 ? timeout : 0) : off;
-
-        if (timeout && notify_listener->needsUpdate ()) {
-            int diff = diffTime (now, last_paint_time);
-            if (!last_paint_time.tv_sec ||
-                    diff > 35 || (timeout > 10 && diff > 25)) {
-                notify_listener->update ();
-                timeOfDay (now);
-                last_paint_time = now;
-                if (notify_listener->needsUpdate () &&
-                        (off == timeout || timeout > 40))
-                    timeout = 40;
-            } else {
-                int next_repaint = 40 - diff;
-                if (off == timeout || next_repaint < timeout)
-                    timeout = next_repaint;
-            }
-        }
-
-        int dtimer = cur_timeout - timeout;
-        addTime (now, timeout);
-        int dtimeout = diffTime (cur_timeout_time, now);
-        bool onoff = (off == timeout && cur_timeout != off) ||
-                (off == cur_timeout && timeout != off);
-        if (onoff || (timeout != off && (-3 > dtimeout || dtimeout > 3))) {
-            if (onoff || (timeout != off && (-3 > dtimer || dtimer > 3))) {
-                cur_timeout = timeout;
-                notify_listener->setTimeout (timeout);
-            }
-            cur_timeout_time = now;
+        timeout = 0x7FFFFFFF != timeout
+            ? (timeout > 0 ? timeout : 0)
+            : TIMEOUT_OFF;
+        if (TIMEOUT_OFF != cur_time_interval && TIMEOUT_OFF != timeout) {
+            if (cur_time_interval > timeout - 5 && cur_time_interval < timeout + 5)
+                return; // too small, let the timer repeat
         }
+        cur_time_interval = timeout;
+        notify_listener->setTimeout (timeout);
     }
 }
 
 void Document::updateTimeout () {
-    if (!cur_event && (-1 == cur_timeout || cur_timeout > 25)) {
+    if (!postpone_ref && event_queue && notify_listener) {
         struct timeval now;
-        timeOfDay (now);
+        if (cur_event)
+            now = cur_event->timeout;
+        else
+            timeOfDay (now);
         setNextTimeout (now);
     }
 }
 
-Event *Document::postEvent (Node *n, Event *e) {
-    int ms = e->id () == event_timer ? static_cast <TimerEvent *> (e)->milli_sec : 0;
+Posting *Document::post (Node *n, Posting *e) {
+    int ms = e->message == MsgEventTimer
+        ? static_cast<TimerPosting *>(e)->milli_sec
+        : 0;
     struct timeval now, tv;
     if (cur_event)
         now = cur_event->timeout;
@@ -1003,31 +1002,32 @@ Event *Document::postEvent (Node *n, Event *e) {
         timeOfDay (now);
     tv = now;
     addTime (tv, ms);
-    insertEvent (n, e, tv);
-    if (postpone_ref || event_queue->event.ptr () == e)
+    insertPosting (n, e, tv);
+    if (postpone_ref || event_queue->event == e)
         setNextTimeout (now);
     return e;
 }
 
-static EventData *findEvent (EventData *queue, EventData **prev, const Event *e) {
+static EventData *findPosting (EventData *queue, EventData **prev, const Posting *e) {
     *prev = NULL;
     for (EventData *ed = queue; ed; ed = ed->next) {
-        if (e == ed->event.ptr ())
+        if (e == ed->event)
             return ed;
         *prev = ed;
     }
     return NULL;
 }
 
-void Document::cancelEvent (Event *e) {
-    if (cur_event && cur_event->event.ptr () == e) {
+void Document::cancelPosting (Posting *e) {
+    if (cur_event && cur_event->event == e) {
+        delete cur_event->event;
         cur_event->event = NULL;
     } else {
         EventData *prev;
         EventData **queue = &event_queue;
-        EventData *ed = findEvent (event_queue, &prev, e);
+        EventData *ed = findPosting (event_queue, &prev, e);
         if (!ed) {
-            ed = findEvent (paused_queue, &prev, e);
+            ed = findPosting (paused_queue, &prev, e);
             queue = &paused_queue;
         }
         if (ed) {
@@ -1044,18 +1044,19 @@ void Document::cancelEvent (Event *e) {
             }
             delete ed;
         } else {
-            errorLog () << "Event not found" << endl;
+            errorLog () << "Posting not found" << endl;
         }
     }
 }
 
-void Document::pauseEvent (Event *e) {
-    if (cur_event && cur_event->event.ptr () == e) {
+void Document::pausePosting (Posting *e) {
+    if (cur_event && cur_event->event == e) {
         paused_queue = new EventData (cur_event->target, cur_event->event, paused_queue);
         paused_queue->timeout = cur_event->timeout;
+        cur_event->event = NULL;
     } else {
         EventData *prev;
-        EventData *ed = findEvent (event_queue, &prev, e);
+        EventData *ed = findPosting (event_queue, &prev, e);
         if (ed) {
             if (prev)
                 prev->next = ed->next;
@@ -1069,34 +1070,34 @@ void Document::pauseEvent (Event *e) {
     }
 }
 
-void Document::unpauseEvent (Event *e, int ms) {
+void Document::unpausePosting (Posting *e, int ms) {
     EventData *prev;
-    EventData *ed = findEvent (paused_queue, &prev, e);
+    EventData *ed = findPosting (paused_queue, &prev, e);
     if (ed) {
         if (prev)
             prev->next = ed->next;
         else
             paused_queue = ed->next;
         addTime (ed->timeout, ms);
-        insertEvent (ed->target, ed->event, ed->timeout);
+        insertPosting (ed->target, ed->event, ed->timeout);
+        ed->event = NULL;
         delete ed;
     } else {
-        errorLog () << "pauseEvent not found";
+        errorLog () << "pausePosting not found" << endl;
     }
 }
 
 void Document::timer () {
     struct timeval now;
-    timeOfDay (now);
     cur_event = event_queue;
-    int diff = cur_event ? diffTime (cur_event->timeout, now) : 0;
-    if (cur_event && diff < 5) {
+    if (cur_event) {
         NodePtrW guard = this;
         struct timeval start = cur_event->timeout;
+        timeOfDay (now);
 
         // handle max 100 timeouts with timeout set to now
         for (int i = 0; i < 100 && active (); ++i) {
-            if (postpone_ref && postponedSensible (cur_event->event.ptr ()))
+            if (postpone_ref && postponedSensible (cur_event->event->message))
                 break;
             // remove from queue
             event_queue = cur_event->next;
@@ -1106,18 +1107,20 @@ void Document::timer () {
                 errorLog () << "spurious timer" << endl;
             } else {
                 EventData *ed = cur_event;
-                cur_event->target->handleEvent (cur_event->event.ptr ());
+                cur_event->target->message (cur_event->event->message, cur_event->event);
                 if (!guard) {
                     delete ed;
                     return;
                 }
-                if (cur_event->event && cur_event->event->id() == event_timer) {
-                    TimerEvent *te = static_cast <TimerEvent *> (cur_event->event.ptr ());
+                if (cur_event->event && cur_event->event->message == MsgEventTimer) {
+                    TimerPosting *te = static_cast <TimerPosting *> (cur_event->event);
                     if (te->interval) {
                         te->interval = false; // reset interval
                         addTime (cur_event->timeout, te->milli_sec);
-                        insertEvent (cur_event->target,
-                                cur_event->event.ptr (), cur_event->timeout);
+                        insertPosting (cur_event->target,
+                                cur_event->event,
+                                cur_event->timeout);
+                        cur_event->event = NULL;
                     }
                 }
             }
@@ -1126,8 +1129,8 @@ void Document::timer () {
             if (!cur_event || diffTime (cur_event->timeout, start) > 5)
                 break;
         }
+        cur_event = NULL;
     }
-    cur_event = NULL;
     setNextTimeout (now);
 }
 
@@ -1137,7 +1140,10 @@ PostponePtr Document::postpone () {
     debugLog () << "postponed " << endl;
     PostponePtr p = new Postpone (this);
     postpone_ref = p;
-    propagateEvent (new PostponedEvent (true));
+    PostponedEvent event (true);
+    deliver (MsgEventPostponed, &event);
+    if (notify_listener)
+        notify_listener->enableRepaintUpdaters (false, 0);
     if (!cur_event) {
         struct timeval now;
         if (event_queue) // save a sys call
@@ -1150,16 +1156,19 @@ PostponePtr Document::postpone () {
 void Document::proceed (const struct timeval &postponed_time) {
     debugLog () << "proceed" << endl;
     postpone_ref = NULL;
+    struct timeval now;
+    timeOfDay (now);
+    int diff = diffTime (now, postponed_time);
     if (event_queue) {
-        struct timeval now;
-        timeOfDay (now);
-        int diff = diffTime (now, postponed_time);
         for (EventData *ed = event_queue; ed; ed = ed->next)
-            if (ed->event && postponedSensible (ed->event))
+            if (ed->event && postponedSensible (ed->event->message))
                 addTime (ed->timeout, diff);
         setNextTimeout (now);
     }
-    propagateEvent (new PostponedEvent (false));
+    if (notify_listener)
+        notify_listener->enableRepaintUpdaters (true, diff);
+    PostponedEvent event (false);
+    deliver (MsgEventPostponed, &event);
 }
 
 Surface *Document::getSurface (Mrl *mrl) {
@@ -1168,10 +1177,13 @@ Surface *Document::getSurface (Mrl *mrl) {
     return NULL;
 }
 
-NodeRefListPtr Document::listeners (unsigned int id) {
-    if (id == event_postponed)
-        return m_PostponedListeners;
-    return Mrl::listeners (id);
+void *Document::message (MessageType msg, void *content) {
+    if (MsgQueryReceivers == msg) {
+        MessageType m = (MessageType) (long) content;
+        if (MsgEventPostponed == m)
+            return m_PostponedListeners.ptr ();
+    }
+    return Mrl::message (msg, content);
 }
 
 //-----------------------------------------------------------------------------
@@ -1254,6 +1266,12 @@ bool GenericMrl::expose () const {
 
 //-----------------------------------------------------------------------------
 
+void Visitor::visit (Element *elm) {
+    visit (static_cast <Node *> (elm));
+}
+
+//-----------------------------------------------------------------------------
+
 namespace KMPlayer {
 
 class KMPLAYER_NO_EXPORT DocumentBuilder {
@@ -1630,7 +1648,7 @@ bool SimpleSAXParser::nextToken () {
                         nextToken () && token->token == tok_semi_colon) {
                     //kdDebug () << "char entity found " << prev_token->string << prev_token->string.toInt (0L, 16) << endl;
                     token->token = tok_text;
-                    if (!prev_token->string.hasPrefix (Char ('x')))
+                    if (!prev_token->string.startsWith (Char ('x')))
                         token->string = Char (prev_token->string.toInt ());
                     else
                         token->string = Char (prev_token->string.mid (1).toInt (0L, 16));
@@ -1780,7 +1798,7 @@ bool SimpleSAXParser::readPI () {
 bool SimpleSAXParser::readDTD () {
     //TODO: <!ENTITY ..>
     if (!nextToken ()) return false;
-    if (token->token == tok_text && token->string.hasPrefix (String ("--"))) {
+    if (token->token == tok_text && token->string.startsWith (String ("--"))) {
         m_state = new StateInfo (InComment, m_state->next); // note: pop DTD
         return readComment ();
     }
@@ -1808,7 +1826,7 @@ bool SimpleSAXParser::readDTD () {
 bool SimpleSAXParser::readCDATA () {
     while (!data->atEnd ()) {
         *data >> next_char;
-        if (next_char == Char ('>') && cdata.hasSuffix (String ("]]"))) {
+        if (next_char == Char ('>') && cdata.endsWith (String ("]]"))) {
             cdata.truncate (cdata.length () - 2);
             m_state = m_state->next;
             if (m_state->state == InContent)
@@ -1830,7 +1848,7 @@ bool SimpleSAXParser::readCDATA () {
 bool SimpleSAXParser::readComment () {
     while (nextToken ()) {
         if (token->token == tok_angle_close && prev_token)
-            if (prev_token->string.hasSuffix (String ("--"))) {
+            if (prev_token->string.endsWith (String ("--"))) {
                 m_state = m_state->next;
                 return true;
             }
@@ -1927,7 +1945,7 @@ bool SimpleSAXParser::parse (TextStream & d) {
                     } else {
                         if (!white_space.isEmpty ()) {
                             if (!in_character_data) {
-                                int pos = white_space.findRev (Char ('\n'));
+                                int pos = white_space.lastIndexOf (Char ('\n'));
                                 if (pos > -1)
                                     white_space = white_space.mid (pos + 1);
                             }
