@@ -35,13 +35,13 @@ namespace KMPlayer {
 
 class Document;
 class Node;
-class Event;
+class Posting;
 class Mrl;
 class Surface;
 class ElementPrivate;
 class Visitor;
-class MediaManager;
-class MediaObject;
+class ActorAgent;
+class Actor;
 
 /*
  * Base class for objects that will be used as SharedPtr/WeakPtr pointers.
@@ -70,7 +70,7 @@ private:
  * Because of the m_self member of Item<T>, it's not allowed to assign a
  * Item<T>* directly to SharedPtr<Item<T>>. Item<T>* will then reside in
  * two independent SharedData<Item<T>> objects.
- * So specialize constructor and assignment operators to fetch the 
+ * So specialize constructor and assignment operators to fetch the
  * SharedData<Item<T>> from the Item<T>* instead of creating a new one
  */
 #define ITEM_AS_POINTER(CLASS)                                         \
@@ -115,7 +115,7 @@ template <class T>
 class KMPLAYER_EXPORT List : public Item <List <T> > {
 public:
     List () {}
-    List (typename Item<T>::SharedType f, typename Item<T>::SharedType l) 
+    List (typename Item<T>::SharedType f, typename Item<T>::SharedType l)
         : m_first (f), m_last (l) {}
     ~List () { clear (); }
 
@@ -215,23 +215,44 @@ enum Fit {
     fit_scroll     // keep aspect and don't scale, add scollbars if needed
 };
 
-extern const unsigned int event_pointer_clicked;
-extern const unsigned int event_pointer_moved;
-extern const unsigned int event_inbounds;
-extern const unsigned int event_outbounds;
-extern const unsigned int event_postponed;
-extern const unsigned int event_timer;
-extern const unsigned int event_started;
-extern const unsigned int event_stopped;
-extern const unsigned int mediatype_attached;
+enum MessageType
+{
+    MsgEventTimer = 0,
+    MsgEventClicked,
+    MsgEventPointerMoved,
+    MsgEventPointerInBounds,
+    MsgEventPointerOutBounds,
+    MsgEventStarting,
+    MsgEventStarted,
+    MsgEventStopped,
+    MsgMediaFinished,
+    // the above messages are ordered like SMIL timing events
+    MsgMediaReady,
+    MsgMediaUpdated,
+    MsgEventPostponed,
+    MsgSurfaceUpdate,
+    MsgSurfaceAttach,
+    MsgStateFreeze,
+    MsgChildFinished,
+    MsgStartQueryMessage,
+    MsgQueryRoleTiming,
+    MsgQueryRoleDisplay,
+    MsgQueryRoleSizer,
+    MsgQueryReceivers,
+    MsgEndQueryMessage,
+};
+
+#define MsgUnhandled ((void *) 357L)
+
+#define nodeMessageReceivers(node, msg)                                     \
+    (NodeRefList*)(node)->message (MsgQueryReceivers, (void*)(long)(msg))
 
 // convenient types
+typedef void Role;
 typedef Item<Node>::SharedType NodePtr;
 typedef Item<Node>::WeakType NodePtrW;
 typedef Item<Attribute>::SharedType AttributePtr;
 typedef Item<Attribute>::WeakType AttributePtrW;
-typedef Item<Event>::SharedType EventPtr;
-typedef Item<Event>::WeakType EventPtrW;
 typedef List<Node> NodeList;                 // eg. for Node's children
 typedef Item<NodeList>::SharedType NodeListPtr;
 typedef Item<NodeList>::WeakType NodeListPtrW;
@@ -244,7 +265,7 @@ ITEM_AS_POINTER(KMPlayer::NodeRefItem)
 //typedef ListNode<NodePtr> NodeStoreItem;   // list stores Nodes
 typedef NodeRefItem::SharedType NodeRefItemPtr;
 typedef NodeRefItem::WeakType NodeRefItemPtrW;
-typedef List<NodeRefItem> NodeRefList;       // ref nodes, eg. event listeners
+typedef List<NodeRefItem> NodeRefList;       // ref nodes, eg. event receivers
 typedef Item<NodeRefList>::SharedType NodeRefListPtr;
 typedef Item<NodeRefList>::WeakType NodeRefListPtrW;
 ITEM_AS_POINTER(KMPlayer::NodeRefList)
@@ -252,7 +273,7 @@ typedef Item<Surface>::SharedType SurfacePtr;
 typedef Item<Surface>::WeakType SurfacePtrW;
 
 /*
- * Weak ref of the listeners list from signaler and the listener node
+ * Weak ref of the receivers list from signaler and the listener node
  */
 class KMPLAYER_EXPORT Connection {
     friend class Node;
@@ -261,7 +282,7 @@ public:
     void disconnect ();
     NodePtrW connectee; // the one that will, when ever, trigger the event
 private:
-    Connection (NodeRefListPtr ls, NodePtr node, NodePtr invoker);
+    Connection (NodeRefList *ls, Node *node, Node *invoker);
     NodeRefListPtrW listeners;
     NodeRefItemPtrW listen_item;
 };
@@ -332,25 +353,21 @@ public:
      * Return a NULL ptr if event_id is not supported.
      * \sa: Connection::disconnect()
      */
-    ConnectionPtr connectTo (NodePtr node, unsigned int event_id);
+    ConnectionPtr connectTo (Node *node, MessageType msg);
     /*
      * Event send to this node, return true if handled
      */
-    virtual bool handleEvent (Event *event);
+    virtual void *message (MessageType msg, void *content=NULL);
     /*
-     * Dispatch Event to all listeners of event->id()
+     * Dispatch Event to all connectorss of MessageType
      */
-    void propagateEvent (Event *event);
+    void deliver (MessageType msg, void *content);
     /**
      * Alternative to event handling is the Visitor pattern
      */
     virtual void accept (Visitor *);
-    /*
-     * Returns a listener list for event_id, or a null ptr if not supported.
-     */
-    virtual NodeRefListPtr listeners (unsigned int event_id);
     /**
-     * Adds node to call 'handleEvent()' for all events that gets
+     * Adds node to call 'message()' for all events that gets
      * delivered to this node, ignored by default
      */
     virtual Surface *getSurface (Mrl *mrl);
@@ -374,7 +391,7 @@ public:
      */
     virtual void defer ();
     /**
-     * Puts a deferred element in activated again, calls activate() again 
+     * Puts a deferred element in activated again, calls activate() again
      */
     virtual void undefer ();
     /**
@@ -383,11 +400,11 @@ public:
     virtual void begin ();
     /**
      * Sets state to state_finish when >= state_activated.
-     * Notifies parent with a childDone call.
+     * Notifies parent with a MsgChildFinished message.
      */
     virtual void finish ();
     /**
-     * Stops element, sets state to state_deactivated. Calls deactivate() on 
+     * Stops element, sets state to state_deactivated. Calls deactivate() on
      * activated/deferred children. May call childDone() when active() and not
      * finished yet.
      */
@@ -397,16 +414,6 @@ public:
      * state to state_init.
      */
     virtual void reset ();
-    /**
-     * Notification from child that it has began.
-     */
-    virtual void childBegan (NodePtr child);
-    /**
-     * Notification from child that it's finished. Will call deactivate() on
-     * child if it state is state_finished. Call activate() on nexSibling
-     * or deactivate() if there is none.
-     */
-    virtual void childDone (NodePtr child);
     virtual void clear ();
     void clearChildren ();
     void appendChild (NodePtr c);
@@ -467,6 +474,7 @@ public:
     virtual void reset ();
     virtual void clear ();
     virtual bool isElementNode () { return true; }
+    virtual void accept (Visitor * v);
     /**
      * Params are like attributes, but meant to be set dynamically. Caller may
      * pass a modification id, that it can use to restore the old value.
@@ -535,14 +543,13 @@ public:
      * By default support one event handler (eg. SMIL or RP child document)
      */
     virtual Surface *getSurface (Mrl *mrl);
-    virtual bool handleEvent (Event *event);
 
     /**
      * If this Mrl is top node of external document, opener has the
      * location in SCR. Typically that's the parent of this node.
      */
     NodePtrW opener; //if this node is top node of external document,
-    MediaObject *media_object;
+    Actor *actor;
     String src;
     String pretty_name;
     String mimetype;
@@ -563,7 +570,7 @@ class KMPLAYER_EXPORT PlayListNotify {
 public:
     virtual ~PlayListNotify () {}
 
-    virtual MediaManager *mediaManager () const = 0;
+    virtual ActorAgent *actorAgent () const = 0;
     /**
      * Called by an unresolved Mrl, check if this node points to a playlist
      */
@@ -592,8 +599,12 @@ public:
      * Request to open url with mimetype
      */
     virtual void openUrl (const URL &, const String &tgt, const String &srv)=0;
-    virtual bool needsUpdate () const = 0;
-    virtual void update () = 0;
+    /**
+     * Change repaint updaters
+     */
+    virtual void addRepaintUpdater (Node *node)=0;
+    virtual void removeRepaintUpdater (Node *node)=0;
+    virtual void enableRepaintUpdaters (bool enable, unsigned int off_time)=0;
 };
 
 class KMPLAYER_NO_EXPORT Surface : public TreeNode <Surface> {
@@ -606,7 +617,6 @@ public:
     virtual void resize (const SRect & rect) = 0;
     virtual void repaint () = 0;
     virtual void repaint (const SRect &rect) = 0;
-    virtual void video (Mrl *) = 0;
     void remove ();                // remove from parent, mark ancestors dirty
     void markDirty ();             // mark this and ancestors dirty
 
@@ -623,36 +633,39 @@ protected:
 ITEM_AS_POINTER(KMPlayer::Surface)
 
 /*
- * A generic event type
- */
-class KMPLAYER_EXPORT Event : public Item <Event> {
+ *  A generic type for posting messages
+ **/
+class KMPLAYER_EXPORT Posting {
 public:
-    KDE_NO_CDTOR_EXPORT Event (Node *n, unsigned int event_id)
-        : source (n), m_event_id (event_id) {}
-    KDE_NO_CDTOR_EXPORT virtual ~Event () {}
-    KDE_NO_EXPORT unsigned int id () const { return m_event_id; }
+    KDE_NO_CDTOR_EXPORT Posting (Node *n, MessageType msg)
+        : source (n), message (msg) {}
+    KDE_NO_CDTOR_EXPORT virtual ~Posting () {}
     NodePtrW source;
-protected:
-    unsigned int m_event_id;
+    MessageType message;
 };
 
-ITEM_AS_POINTER(KMPlayer::Event)
-
 /**
- * Event signaling a timer event
+ * Posting signaling a timer event
  */
-class KMPLAYER_NO_EXPORT TimerEvent : public Event {
+class KMPLAYER_NO_EXPORT TimerPosting : public Posting {
 public:
-    TimerEvent (int ms, unsigned eid=0);
+    TimerPosting (int ms, unsigned eid=0);
     unsigned event_id;
     int milli_sec;
-    bool interval; // set to 'true' in 'Node::handleEvent()' to make it repeat
+    bool interval; // set to 'true' in 'Node::message()' to make it repeat
+};
+
+class KMPLAYER_NO_EXPORT UpdateEvent {
+public:
+    UpdateEvent (Document *, unsigned int off_time);
+    unsigned int cur_event_time;
+    unsigned int skipped_time;
 };
 
 /**
  * Event signaling postponed or proceeded
  */
-class KMPLAYER_NO_EXPORT PostponedEvent : public Event {
+class KMPLAYER_NO_EXPORT PostponedEvent {
 public:
     PostponedEvent (bool postponed);
     bool is_postponed; // postponed or proceeded
@@ -675,10 +688,11 @@ typedef SharedPtr <Postpone> PostponePtr;
 typedef WeakPtr <Postpone> PostponePtrW;
 
 struct KMPLAYER_NO_EXPORT EventData {
-    EventData (Node *t, Event *e, EventData *n);
+    EventData (Node *t, Posting *e, EventData *n);
+    ~EventData ();
 
     NodePtrW target;
-    EventPtr event;
+    Posting *event;
     struct timeval timeout;
 
     EventData *next;
@@ -705,10 +719,10 @@ public:
     virtual void undefer ();
     virtual void reset ();
 
-    Event *postEvent (Node *n, Event *event);
-    void cancelEvent (Event *event);
-    void pauseEvent (Event *e);
-    void unpauseEvent (Event *e, int ms);
+    Posting *post (Node *n, Posting *event);
+    void cancelPosting (Posting *event);
+    void pausePosting (Posting *e);
+    void unpausePosting (Posting *e, int ms);
 
     void timeOfDay (struct timeval &);
     PostponePtr postpone ();
@@ -719,9 +733,9 @@ public:
     void timer ();
     void updateTimeout ();
     /**
-     * Document has list of postponed listeners, eg. for running (gif)movies
+     * Document has list of postponed receivers, eg. for running (gif)movies
      */
-    virtual NodeRefListPtr listeners (unsigned int event_id);
+    virtual void *message (MessageType msg, void *content=NULL);
     /**
      * Reimplement, so it will call PlayListNotify::getSurface()
      */
@@ -732,7 +746,7 @@ public:
     unsigned int last_event_time;
 private:
     void proceed (const struct timeval & postponed_time);
-    void insertEvent (Node *n, Event *e, const struct timeval &tv);
+    void insertPosting (Node *n, Posting *e, const struct timeval &tv);
     void setNextTimeout (const struct timeval &now);
 
     PostponePtrW postpone_ref;
@@ -741,10 +755,8 @@ private:
     EventData *event_queue;
     EventData *paused_queue;
     EventData *cur_event;
-    int cur_timeout;
     struct timeval first_event_time;
-    struct timeval last_paint_time;
-    struct timeval cur_timeout_time;
+    int cur_time_interval;
 };
 
 /**
@@ -789,9 +801,11 @@ protected:
 
 namespace SMIL {
     class Smil;
-    class RegionBase;
-    class Region;
     class Layout;
+    class RegionBase;
+    class Seq;
+    class Par;
+    class Excl;
     class Transition;
     class Animate;
     class AnimateMotion;
@@ -801,7 +815,6 @@ namespace SMIL {
     class RefMediaType;
     class AVMediaType;
     class Brush;
-    class TimedMrl;
     class PriorityClass;
     class Anchor;
     class Area;
@@ -822,13 +835,16 @@ public:
     KDE_NO_CDTOR_EXPORT Visitor () {}
     KDE_NO_CDTOR_EXPORT virtual ~Visitor () {}
     virtual void visit (Node *) {}
+    virtual void visit (Element *);
     virtual void visit (SMIL::Smil *) {}
-    virtual void visit (SMIL::Region *);
     virtual void visit (SMIL::Layout *);
+    virtual void visit (SMIL::RegionBase *);
+    virtual void visit (SMIL::Seq *);
+    virtual void visit (SMIL::Par *);
+    virtual void visit (SMIL::Excl *);
     virtual void visit (SMIL::Transition *);
     virtual void visit (SMIL::Animate *);
     virtual void visit (SMIL::AnimateMotion *);
-    virtual void visit (SMIL::TimedMrl *);
     virtual void visit (SMIL::PriorityClass *);
     virtual void visit (SMIL::MediaType *);
     virtual void visit (SMIL::ImageMediaType *);
