@@ -24,10 +24,16 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
 
+#include <gtk/gtk.h>
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
 
+#include "kmplayer.h"
 #include "kmplayerprocess.h"
 #ifdef __ARMEL__
 # include "npplayer.h"
@@ -75,7 +81,7 @@ Process::~Process () {
 //    return 0;
 //}
 
-bool Process::playing () const {
+KMPLAYER_NO_EXPORT bool Process::playing () const {
     return m_state > Ready;
 }
 
@@ -83,40 +89,40 @@ bool Process::playing () const {
 
 //void Process::setSubtitle (int, const QString &) {}
 
-void Process::setPosition (int p) {
+KMPLAYER_NO_EXPORT void Process::setPosition (int p) {
     m_position = p;
     m_notify->setPosition (m_position);
 }
 
-bool Process::pause () {
+KMPLAYER_NO_EXPORT bool Process::pause () {
     return false;
 }
 
-bool Process::seek (int /*pos*/, bool /*absolute*/) {
+KMPLAYER_NO_EXPORT bool Process::seek (int /*pos*/, bool /*absolute*/) {
     return false;
 }
 
-bool Process::volume (int /*pos*/, bool /*absolute*/) {
+KMPLAYER_NO_EXPORT bool Process::volume (int /*pos*/, bool /*absolute*/) {
     return false;
 }
 
-bool Process::saturation (int /*pos*/, bool /*absolute*/) {
+KMPLAYER_NO_EXPORT bool Process::saturation (int /*pos*/, bool /*absolute*/) {
     return false;
 }
 
-bool Process::hue (int /*pos*/, bool /*absolute*/) {
+KMPLAYER_NO_EXPORT bool Process::hue (int /*pos*/, bool /*absolute*/) {
     return false;
 }
 
-bool Process::contrast (int /*pos*/, bool /*absolute*/) {
+KMPLAYER_NO_EXPORT bool Process::contrast (int /*pos*/, bool /*absolute*/) {
     return false;
 }
 
-bool Process::brightness (int /*pos*/, bool /*absolute*/) {
+KMPLAYER_NO_EXPORT bool Process::brightness (int /*pos*/, bool /*absolute*/) {
     return false;
 }
 
-void Process::setVideoWindow (int, int, int, int) {}
+KMPLAYER_NO_EXPORT void Process::setVideoWindow (int, int, int, int) {}
 
 //bool Process::grabPicture (const KURL & /*url*/, int /*pos*/) {
 //    return false;
@@ -130,12 +136,12 @@ void Process::setVideoWindow (int, int, int, int) {}
 //    return false;
 //}
 
-bool Process::stop () {
+KMPLAYER_NO_EXPORT bool Process::stop () {
     if (!playing ()) return true;
     return false;
 }
 
-bool Process::quit () {
+KMPLAYER_NO_EXPORT bool Process::quit () {
     debugLog() << "quit" << endl;
     stop ();
     setState (NotRunning);
@@ -147,7 +153,7 @@ static gboolean cb_scheduledUpdateAspects (void * p) {
     return false; // single shot
 }
 
-void Process::updateAspects () {
+KMPLAYER_NO_EXPORT void Process::updateAspects () {
     if (m_notify)
         m_notify->setAspect (m_height > 0 ? 1.0 * m_width / m_height : 0);
 }
@@ -157,7 +163,7 @@ static gboolean cb_scheduledStateChanged (void * p) {
     return false; // single shot
 }
 
-void Process::setState (State newstate) {
+KMPLAYER_NO_EXPORT void Process::setState (State newstate) {
     if (m_state != newstate) {
         m_old_state = m_state;
         m_state = newstate;
@@ -165,7 +171,7 @@ void Process::setState (State newstate) {
     }
 }
 
-void Process::scheduledStateChanged () {
+KMPLAYER_NO_EXPORT void Process::scheduledStateChanged () {
     m_notify->stateChanged (this, m_old_state, m_state);
     start_timer = 0;
 }
@@ -180,13 +186,13 @@ KMPLAYER_NO_EXPORT bool Process::play (NodePtr _mrl) {
     return false;
 }
 
-bool Process::ready (unsigned int viewer) {
+KMPLAYER_NO_EXPORT bool Process::ready (unsigned int viewer) {
     m_viewer = viewer;
     setState (Ready);
     return true;
 }
 
-unsigned int Process::viewer () const {
+KMPLAYER_NO_EXPORT unsigned int Process::viewer () const {
     return m_viewer;
 }
 
@@ -224,7 +230,7 @@ MPlayer::MPlayer (ProcessNotify * ctrl, bool xv)
 MPlayer::~MPlayer () {
 }
 
-bool MPlayer::play (NodePtr node) {
+KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     debugLog () << "MPlayer::play" << endl;
     if (playing ())
         return pause ();
@@ -248,7 +254,7 @@ bool MPlayer::play (NodePtr node) {
     if (url.isLocalFile ()) {
         strurl = url.path ();
         wd = url.path ();
-        int pos = wd.findRev (String ("/"));
+        int pos = wd.lastIndexOf (String ("/"));
         if (pos > -1)
             wd.truncate (pos);
     }
@@ -310,29 +316,29 @@ bool MPlayer::play (NodePtr node) {
     return false;
 }
 
-bool MPlayer::pause () {
+KMPLAYER_NO_EXPORT bool MPlayer::pause () {
     wait_for_pause = !wait_for_pause;
     debugLog() << "MPlayer::pause schedule " << wait_for_pause << endl;
     for (StringList::iterator i = commands.begin (); i != commands.end (); ++i)
-        if (i->hasPrefix ("pause")) {
+        if (i->startsWith ("pause")) {
             commands.erase (i);
             return true;
         }
     return sendCommand (String ("pause"));
 }
 
-bool MPlayer::ready (unsigned int viewer) {
+KMPLAYER_NO_EXPORT bool MPlayer::ready (unsigned int viewer) {
     if (m_state < Ready)
         return Process::ready (viewer);
     return false;
 }
 
-bool MPlayer::stop () {
+KMPLAYER_NO_EXPORT bool MPlayer::stop () {
     debugLog() << "stop mplayer" << endl;
     return sendCommand (String ("quit"));
 }
 
-bool MPlayer::quit () {
+KMPLAYER_NO_EXPORT bool MPlayer::quit () {
     if (playing ()) {
         request_quit = true;
         stop ();
@@ -374,12 +380,12 @@ bool MPlayer::quit () {
     return true;
 }
 
-bool MPlayer::seek (int pos, bool absolute) {
+KMPLAYER_NO_EXPORT bool MPlayer::seek (int pos, bool absolute) {
     if (/*request_seek != NO_SEEK &&*/ commands.size () > 1) {
         StringList::iterator i = commands.begin ();
         StringList::iterator end ( commands.end () );
         for (++i; i != end; ++i)
-            if ((*i).hasPrefix (String ("seek"))) {
+            if ((*i).startsWith (String ("seek"))) {
                 i = commands.erase (i);
                 request_seek = NO_SEEK;
                 break;
@@ -397,27 +403,27 @@ bool MPlayer::seek (int pos, bool absolute) {
     return sendCommand (String (buf));
 }
 
-bool MPlayer::volume (int pos, bool /*absolute*/) {
+KMPLAYER_NO_EXPORT bool MPlayer::volume (int pos, bool /*absolute*/) {
     return sendCommand (String ("volume ") + String::number (pos));
 }
 
-bool MPlayer::saturation (int pos, bool absolute) {
+KMPLAYER_NO_EXPORT bool MPlayer::saturation (int pos, bool absolute) {
     return true;
 }
 
-bool MPlayer::hue (int pos, bool absolute) {
+KMPLAYER_NO_EXPORT bool MPlayer::hue (int pos, bool absolute) {
     return true;
 }
 
-bool MPlayer::contrast (int pos, bool absolute) {
+KMPLAYER_NO_EXPORT bool MPlayer::contrast (int pos, bool absolute) {
     return true;
 }
 
-bool MPlayer::brightness (int pos, bool absolute) {
+KMPLAYER_NO_EXPORT bool MPlayer::brightness (int pos, bool absolute) {
     return true;
 }
 
-void MPlayer::setVideoWindow (int x, int y, int w, int h) {
+KMPLAYER_NO_EXPORT void MPlayer::setVideoWindow (int x, int y, int w, int h) {
     debugLog() << "setVideoWindow " << w << " " << h << endl;
     if (x < 0 || y < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
                 x > 800 || y > 480 || w > 800 || h > 480)
@@ -431,7 +437,7 @@ void MPlayer::setVideoWindow (int x, int y, int w, int h) {
     sendCommand (String (buf));
 }
 
-void MPlayer::processExited (GPid pid) {
+KMPLAYER_NO_EXPORT void MPlayer::processExited (GPid pid) {
     if (pid == process_id) {
         process_id = 0;
         commands.clear ();
@@ -452,7 +458,7 @@ void MPlayer::processExited (GPid pid) {
     }
 }
 
-void MPlayer::writeEvent () {
+KMPLAYER_NO_EXPORT void MPlayer::writeEvent () {
     if (!pin)
         return;
     String data;
@@ -479,7 +485,7 @@ void MPlayer::writeEvent () {
     }
 }
 
-void MPlayer::readEvent (GIOChannel * src) {
+KMPLAYER_NO_EXPORT void MPlayer::readEvent (GIOChannel * src) {
     gsize nr;
     int inbuf = strlen (outbuf);
     g_io_channel_read_chars (src, outbuf+inbuf, sizeof (outbuf)-inbuf, &nr, 0L);
@@ -582,7 +588,7 @@ void MPlayer::readEvent (GIOChannel * src) {
     }
 }
 
-bool MPlayer::sendCommand (const String & cmd) {
+KMPLAYER_NO_EXPORT bool MPlayer::sendCommand (const String & cmd) {
     if (playing ()) {
         commands.push_back (cmd + Char ('\n'));
         if (!win)
@@ -600,7 +606,7 @@ static const char *media_server_imusic = "com.nokia.osso_media_server.music";
 static const char *media_server_ivideo = "com.nokia.osso_media_server.video";
 static const char *media_server_filter = "type='signal',sender='com.nokia.osso_media_server'";
 
-gboolean cb_stoppedTimer (void * p) {
+static gboolean cb_stoppedTimer (void * p) {
     OssoMediaServer * process = static_cast <OssoMediaServer *> (p);
     process->processStopped ();
     return 0;
@@ -973,7 +979,7 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::retry () {
         return false;
     } else if (retry_timer) {
         return true;
-    } else if (m_mrl && !mrl ()->audio_only && mrl ()->media_object) {
+    } else if (m_mrl && !mrl ()->audio_only && mrl ()->actor) {
         try_audio_only = true;
         retry_timer = g_timeout_add (100, cb_retryAudioOnly, this);
         return true;
@@ -1406,7 +1412,7 @@ KMPLAYER_NO_EXPORT void NpPlayer::requestStream (void *inst, uint32_t sid,
     debugLog () << "NpPlayer::request " << sid << " '" << uri << "'" << endl;
     if (!target.isEmpty ()) {
         debugLog () << "new page request " << target << endl;
-        if (url.hasPrefix ("javascript:")) {
+        if (url.startsWith ("javascript:")) {
             String result = evaluateScript (url.mid (11));
             debugLog() << "result is " << result << endl;
             if (result == "undefined")
@@ -1417,7 +1423,7 @@ KMPLAYER_NO_EXPORT void NpPlayer::requestStream (void *inst, uint32_t sid,
         /*if (!uri.url ().isEmpty ())
             emit openUrl (uri, target);*/
         nppStreamFinished (sid, NpStream::BecauseDone, this);
-    } else if (url.hasPrefix ("javascript:")) {
+    } else if (url.startsWith ("javascript:")) {
         String result = evaluateScript (url.mid (11));
         const char *data = (const char *)result;
         nppStreamData (sid, data, strlen (data), this);
@@ -1531,7 +1537,7 @@ KDE_NO_CDTOR_EXPORT NpStream::~NpStream () {
 
 KMPLAYER_NO_EXPORT void NpStream::open () {
     debugLog () << "NpStream::open " << url.url () << endl;
-    if (url.url().hasPrefix ("javascript:")) {
+    if (url.url().startsWith ("javascript:")) {
         String result = player->evaluateScript (url.url().mid (11));
         const char *data = (const char *)result;
         nppStreamData (stream, data, strlen (data), NULL /*FIXME*/);
@@ -1570,3 +1576,67 @@ KMPLAYER_NO_EXPORT void NpStream::redirected (Job *, const String & uri) {
 }
 
 #endif //__ARMEL__
+
+Downloader::Downloader (ProcessNotify *ctrl, Application *app)
+ : Process (ctrl, "downloader", false),
+   job (NULL),
+   application (app),
+   fd (-1) {
+}
+
+Downloader::~Downloader () {
+}
+
+bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
+    Process::play (mrl);
+    size = bytes = 0;
+    saved_file.clear ();
+    if (!m_url.isEmpty ()) {
+        gchar *filename = NULL;
+        filename = app_file_chooser (application, GTK_FILE_CHOOSER_ACTION_SAVE);
+        if (filename) {
+            fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0664);
+            if (fd > 0) {
+                job = asyncGet (this, m_url);
+                setState (Buffering);
+                saved_file = filename;
+                g_free (filename);
+                return true;
+            }
+        }
+    }
+    setState (NotRunning);
+    return false;
+}
+
+KMPLAYER_NO_EXPORT bool Downloader::stop () {
+    if (job)
+        job->kill ();
+    if (fd > 0)
+        close (fd);
+    job = NULL;
+    fd = -1;
+    setState (NotRunning);
+    return true;
+}
+
+KMPLAYER_NO_EXPORT void Downloader::jobData (Job *jb, ByteArray & data) {
+    if (!bytes)
+        size = jb->contentLength ();
+    bytes += data.size ();
+    if (size > 0)
+        m_notify->setLoading ((int) (100.0 * bytes / size));
+    if (data.size ())
+        write (fd, data.data (), data.size ());
+}
+
+KMPLAYER_NO_EXPORT void Downloader::jobResult (Job *) {
+    m_notify->setLoading (100);
+    if (fd > 0)
+        close (fd);
+    job = NULL;
+    fd = -1;
+    if (bytes == 0)
+        saved_file.clear ();
+    setState (NotRunning);
+}
