@@ -181,7 +181,7 @@ GLibString GLibString::stripWhiteSpace () const {
     return str;
 }
 
-bool GLibString::hasPrefix (const GLibString & prefix) const {
+bool GLibString::startsWith (const GLibString & prefix) const {
     if (!prefix.data && !data)
         return true;
     if (prefix.data && data)
@@ -189,7 +189,7 @@ bool GLibString::hasPrefix (const GLibString & prefix) const {
     return !prefix.data;
 }
     
-bool GLibString::hasSuffix (const GLibString & suffix) const {
+bool GLibString::endsWith (const GLibString & suffix) const {
     if (!suffix.data && !data)
         return true;
     if (suffix.data && data)
@@ -197,7 +197,7 @@ bool GLibString::hasSuffix (const GLibString & suffix) const {
     return !suffix.data;
 }
 
-int GLibString::find (const GLibString & needle) const {
+int GLibString::indexOf (const GLibString & needle) const {
     if (!data)
         return needle.data ? -1 : 0;
     if (!needle.data)
@@ -208,7 +208,7 @@ int GLibString::find (const GLibString & needle) const {
     return int (str - data->ptr);
 }
 
-int GLibString::findRev (const GLibString & needle) const {
+int GLibString::lastIndexOf (const GLibString & needle) const {
     if (!data)
         return needle.data ? -1 : 0;
     if (!needle.data)
@@ -311,7 +311,7 @@ void GLibString::truncate (int pos) {
 
 //-----------------------------------------------------------------------------
 
-namespace {
+namespace KMPlayer {
 
 class GnomeVFSFilePrivate {
 public:
@@ -1040,31 +1040,32 @@ GRect GRect::intersect (const GRect & r) const {
 
 static GdkColor * colorFromRGB (int rgb) {
     GdkColormap * cmap = gdk_colormap_get_system ();
-    GdkColor * color = g_new (GdkColor, 1);
-    char buf [16]; 
+    GdkColor *color = g_new (GdkColor, 1);
+    GdkColor *rcolor = NULL;
+    char buf [16];
     g_sprintf (buf, "#%02X%02X%02X", (rgb & 0x00FF0000) >> 16, (rgb & 0x0000FF00) >> 8, (rgb & 0x000000FF));
-    if (!(gdk_color_parse (buf, color) &&
+    if (gdk_color_parse (buf, color) &&
                 (gdk_colormap_alloc_color (cmap, color, 0, 0) ||
-                 gdk_colormap_alloc_color (cmap, color, 0, 1)))) {
-        g_free (color);
-        color = 0L;
-    }
-    return color;
+                 gdk_colormap_alloc_color (cmap, color, 0, 1)))
+        rcolor = gdk_color_copy (color);
+    g_free (color);
+    return rcolor;
 }
 
-KMPLAYER_NO_CDTOR_EXPORT GColor::GColor (const String & desc) {
+KMPLAYER_NO_CDTOR_EXPORT GColor::GColor (const String & desc) : m_color (NULL) {
     GdkColormap * cmap = gdk_colormap_get_system ();
-    m_color = g_new (GdkColor, 1); 
-    if (!(gdk_color_parse ((const char *) desc, m_color) &&
-                gdk_colormap_alloc_color (cmap, m_color, 0, 1))) {
-        g_free (m_color);
-        m_color = 0L;
+    GdkColor *color = g_new (GdkColor, 1);
+    if (!(gdk_color_parse ((const char *) desc, color) &&
+                gdk_colormap_alloc_color (cmap, color, 0, 1))) {
         if (desc.lower () == "silver")
             m_color = colorFromRGB (0xbebebe);
         // TODO add more unsupported real live colors
         if (!m_color)
             errorLog () << "Could not parse " << desc << " color" << endl;
+    } else {
+        m_color = gdk_color_copy (color);
     }
+    g_free (color);
     //debugLog () << desc << " -> " << int (m_color ? m_color->pixel : 0) << endl;
 }
 
